@@ -6,6 +6,8 @@ from typing import Dict, Callable
 
 log = logging.getLogger(__name__)
 
+_RE_ROUTE = re.compile(r'(?<!^)(?=[A-Z])')
+
 
 class _Req:
 
@@ -21,7 +23,7 @@ def req(method: str, **http_fields):
 
         @functools.wraps(func)
         def req_maker(*args, **kwargs) -> _Req:
-            route = re.sub(r'(?<!^)(?=[A-Z])', '-', func.__qualname__).lower().replace('.', '/')
+            route = _RE_ROUTE.sub('-', func.__qualname__).lower().replace('.', '/')
 
             # dump args into kwargs
             param_names = list(inspect.signature(func).parameters.keys())
@@ -29,7 +31,13 @@ def req(method: str, **http_fields):
                 kwargs[param_names[i]] = args[i]
 
             # merge http_fields with kwargs
-            params = {'data' if method == 'POST' else 'params': kwargs}
+            payload_key = 'params'  # GET: params=
+            if method == 'POST':
+                payload_key = 'json'  # POST: in default json=
+                if http_fields.pop('headers', {}).pop('Content-Type', None) is not None:
+                    payload_key = 'data'  # POST: in special cases(like guild-emoji/create) use data=
+
+            params = {payload_key: kwargs}
             params.update(http_fields)
 
             return _Req(method, route, params)
@@ -358,7 +366,7 @@ class User:
 class Asset:
 
     @staticmethod
-    @req('POST')
+    @req('POST', headers={'Content-Type': 'form-data'})
     def create(file):
         ...
 
