@@ -21,19 +21,22 @@ class HTTPRequester:
     def __del__(self):
         asyncio.get_event_loop().run_until_complete(self._cs.close())
 
-    async def request(self, method: str, route: str, **params) -> Union[dict, list]:
+    async def request(self, method: str, route: str, **params) -> Union[dict, list, bytes]:
         headers = params.pop('headers', {})
-        headers['Authorization'] = f'Bot {self._cert.token}'
         params['headers'] = headers
 
+        log.debug(f'{method} {route}: req: {params}')  # token is excluded
+        headers['Authorization'] = f'Bot {self._cert.token}'
         async with self._cs.request(method, f'{API}/{route}', **params) as res:
-            log.debug(f'req: {method} {route}: {params}')
-            rsp = await res.json()
-            if rsp['code'] != 0:
-                raise HTTPRequester.APIRequestFailed(method, route, params, rsp['code'], rsp['message'])
+            if res.content_type == 'application/json':
+                rsp = await res.json()
+                if rsp['code'] != 0:
+                    raise HTTPRequester.APIRequestFailed(method, route, params, rsp['code'], rsp['message'])
+                rsp = rsp['data']
             else:
-                log.debug(f'req done: {rsp}')
-            return rsp['data']
+                rsp = await res.read()
+            log.debug(f'{method} {route}: rsp: {rsp}')
+            return rsp
 
     async def exec_req(self, r: _Req):
         return await self.request(r.method, r.route, **r.params)
@@ -74,6 +77,12 @@ class HTTPRequester:
         return ret
 
     class APIRequestFailed(Exception):
+        """
+        Raised when khl.py received non-zero error code from remote server. 
+        By default, params (request body) is not included when calling __str__ on it, to avoid leaking credential 
+        into logs; if request body is needed for debug purpose, consider explicitly catching this exception and 
+        call repr(...) with the exception instance.
+        """
 
         def __init__(self, method, route, params, err_code, err_message):
             self.method = method
@@ -81,3 +90,6 @@ class HTTPRequester:
             self.params = params
             self.err_code = err_code
             self.err_message = err_message
+
+        def __str__(self):
+            return f"Requesting '{self.method} {self.route}' failed with {self.err_code}: {self.err_message}"
